#include "DXBuffer.h"
#include "Render3D.h"
#include <iostream>
#include <d3d11.h>

using namespace Framework;

// Inhalt der DXBuffer Klasse

// Konstruktor
//  bind: Der verwendungszweck des Buffers. Beispiel: D3D11_BIND_INDEX_BUFFER, D3D11_BIND_VERTEX_BUFFER.
//  eLn: Lnge eines einzelnen Elements in Bytes
DXBuffer::DXBuffer( D3D11_BIND_FLAG bind, int eLn )
{
    buffer = 0;
    bf = bind;
    data = 0;
    gendert = 0;
    ln = 0;
    altLn = 0;
    elLn = eLn;
    ref = 1;
}

// Destruktor
DXBuffer::~DXBuffer()
{
    if( buffer )
        buffer->Release();
}

// Setzt den gendert flg, so das beim nchsten auruf von 'kopieren' die daten neu kopiert werden
void DXBuffer::setGendert()
{
    gendert = 1;
}

// ndert die lnge des Buffers beim nchsten aufruf von 'kopieren'
//  ln: Die Lnge in Bytes
void DXBuffer::setLnge( int ln )
{
    this->ln = ln;
    gendert = 1;
}

// Legt fest, was beim nchsten aufruf von 'kopieren' kopiert wird
//  data: Ein zeiger auf die Daten
void DXBuffer::setData( void *data )
{
    this->data = data;
    gendert = 1;
}

// Kopiert die Daten in den Buffer, fals sie sich verndert haben
//  zRObj: Das Objekt, mit dem die Grafikkarte angesprochen wird
void DXBuffer::copieren( Render3D *zRObj )
{
    if( !gendert || !ln || !data )
        return;
    if( ln != altLn )
    {
        if( buffer )
            buffer->Release();
        buffer = 0;
    }
    if( !buffer )
    {
        D3D11_BUFFER_DESC desk;
        memset( &desk, 0, sizeof( desk ) );
        desk.Usage = D3D11_USAGE_DYNAMIC;
        desk.ByteWidth = ln;
        desk.BindFlags = bf;
        desk.CPUAccessFlags = D3D11_CPU_ACCESS_WRITE;

        D3D11_SUBRESOURCE_DATA ini;
        memset( &ini, 0, sizeof( ini ) );
        ini.pSysMem = data;

        zRObj->zDevice()->CreateBuffer( &desk, &ini, &buffer );
        altLn = ln;
    }
    else if( gendert )
    {
        D3D11_MAPPED_SUBRESOURCE map;
        zRObj->zContext()->Map( buffer, 0, D3D11_MAP::D3D11_MAP_WRITE_DISCARD, 0, &map );
        memcpy( map.pData, data, ln );
        zRObj->zContext()->Unmap( buffer, 0 );
        gendert = 0;
    }
}

// Gibt die Lnge eines Elementes in bytes zurck
int DXBuffer::getElementLnge() const
{
    return elLn;
}

// Gibt den Buffer zurck
ID3D11Buffer *DXBuffer::zBuffer() const
{
    return buffer;
}

// Gibt die Anzahl der Elemente im Buffer zurck
int DXBuffer::getElementAnzahl() const
{
    return altLn / elLn;
}

// Erhht den Reference Counting Zhler.
//  return: this.
DXBuffer *DXBuffer::getThis()
{
    ref++;
    return this;
}

// Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
//  return: 0.
DXBuffer *DXBuffer::release()
{
    ref--;
    if( !ref )
        delete this;
    return 0;
}


// Inhalt der DXVertexBuffer Klasse

// Konstruktor
// eSize: Die Lnge eines Elementes in Bytes
DXVertexBuffer::DXVertexBuffer( int eSize )
    : DXBuffer( D3D11_BIND_VERTEX_BUFFER, eSize )
{
}

// Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
//  return: 0.
DXBuffer *DXVertexBuffer::release()
{
    ref--;
    if( !ref )
        delete this;
    return 0;
}


// Inhalt der DXIndexBuffer Klasse

// Konstruktor
// eSize: Die Lnge eines Elementes in Bytes
DXIndexBuffer::DXIndexBuffer( int eSize )
    : DXBuffer( D3D11_BIND_INDEX_BUFFER, eSize )
{
}

// Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
//  return: 0.
DXBuffer *DXIndexBuffer::release()
{
    ref--;
    if( !ref )
        delete this;
    return 0;
}