#include "Schluessel.h"

using namespace Framework::Verschlsselung;

// Inhalt der Bytes Klasse aus Schlssel.h
// Konstruktor
Bytes::Bytes()
	: bytes( 0 ),
	  del( 1 ),
	  ln( 0 ),
	  ref( 1 )
{
}

Bytes::Bytes( int ln )
	: bytes( new char[ ln ] ),
	  del( 1 ),
	  ln( ln ),
	  ref( 1 )
{
}

Bytes::Bytes( const char *daten, int ln )
	: bytes( new char[ ln ] ),
	  del( 1 ),
	  ln( ln ),
	  ref( 1 )
{
	setBytes( daten );
}

// Destruktor
Bytes::~Bytes()
{
	if( del )
	    delete[] bytes;
}

// nicht constant
void Bytes::setBytes( const char *daten )
{
	if( !bytes || !daten )
		return;
	char *end = bytes + ln;
	for( char *c = bytes; c < end; c++, ++daten )
		*c = *daten;
}

void Bytes::setBytes( const char *daten, int ln )
{
	if( !daten || !ln )
		return;
	if( del )
	    delete[] bytes;
	del = 1;
	bytes = new char[ ln ];
	this->ln = ln;
	setBytes( daten );
}

void Bytes::setBytesZ( char *daten, int ln )
{
	if( del )
	    delete[] bytes;
	del = 0;
	bytes = daten;
	this->ln = ln;
}

void Bytes::fll( const char c )
{
	if( !bytes )
		return;
	char *end = bytes + ln;
	for( char *b = bytes; b < end; ++b )
		*b = c;
}

void Bytes::fll( const char c, int ln )
{
	if( !bytes )
		bytes = new char[ ln ];
	ln = ln > this->ln ? this->ln : ln;
	char *end = bytes + ln;
	for( char *b = bytes; b < end; ++b )
		*b = c;
}

void Bytes::fll( const char c, int beg, int end )
{
	if( beg >= ln )
		return;
	if( !bytes )
		bytes = new char[ end - beg ];
	end = end > this->ln ? this->ln : end;
	char *endB = bytes + end;
	for( char *b = bytes + beg; b < endB; ++b )
		*b = c;
}

void Bytes::fll( const char *c, int cLn )
{
	if( !c || !cLn || !bytes )
		return;
	char *endB = bytes + ln;
	const char *endC = c + cLn;
	const char *d = c;
	for( char *b = bytes; b < endB; b++, d = d < endC - 1 ? d + 1 : c )
		*b = *d;
}

void Bytes::set( const char c, int pos )
{
	if( !bytes || pos >= ln )
		return;
	bytes[ pos ] = c;
}

// constant
int Bytes::getLnge() const
{
	return ln;
}

char *Bytes::getBytes() const
{
	return bytes;
}

// Reference Counting
Bytes *Bytes::getThis()
{
	++ref;
	return this;
}

Bytes *Bytes::release()
{
	--ref;
	if( !ref )
		delete this;
	return 0;
}


// Inhalt der Schlssel Klasse aus Schlssel.h
// Konstruktor
Schlssel::Schlssel()
	: schlssel( 0 ),
	  ln( 0 ),
	  pos( 0 ),
	  ref( 1 )
{
}

Schlssel::Schlssel( const char *s, int ln )
	: schlssel( new unsigned char[ ln ] ),
	  ln( ln ),
	  pos( 0 ),
	  ref( 1 )
{
	for( int i = 0; i < ln; ++i )
		schlssel[ i ] = s[ i ];
}

// Destruktor
Schlssel::~Schlssel()
{
	delete[] schlssel;
}

// nicht constant
void Schlssel::setPos( int p )
{
	if( p < 0 )
		p = 0;
	pos = p > ln ? 0 : p;
}

void Schlssel::setSchlssel( const char *s, int ln )
{
	delete[] schlssel;
	schlssel = new unsigned char[ ln ];
	for( int i = 0; i < ln; ++i )
		schlssel[ i ] = s[ i ];
	pos = 0;
	this->ln = ln;
}

void Schlssel::codieren( Bytes *daten )
{
	if( !schlssel || !ln )
	{
		daten->release();
		return;
	}
	int dLn = daten->getLnge();
	char *bytes = daten->getBytes();
	char *bEnd = bytes + dLn;
	for( char *c = bytes; c < bEnd; ++c )
	{
		*c = *c + schlssel[ pos ];
		++pos;
		if( pos >= ln )
			pos = 0;
	}
	daten->release();
}

void Schlssel::decodieren( Bytes *daten )
{
	if( !schlssel || !ln )
	{
		daten->release();
		return;
	}
	int dLn = daten->getLnge();
	char *bytes = daten->getBytes();
	char *bEnd = bytes + dLn;
	for( char *c = bytes; c < bEnd; ++c )
	{
		*c = *c - schlssel[ pos ];
		++pos;
		if( pos >= ln )
			pos = 0;
	}
	daten->release();
}

// Reference Counting
Schlssel *Schlssel::getThis()
{
	++ref;
	return this;
}

Schlssel *Schlssel::release()
{
	--ref;
	if( !ref )
		delete this;
	return 0;
}