#ifndef Schluessel_H
#define Schluessel_H

#include "Betriebssystem.h"

namespace Framework
{
	namespace Verschlsselung
	{
        // Speichert einen Array von bytes
		class Bytes
		{
		private:
			char *bytes;
			bool del;
			int ln;
			int ref;

		public:
			// Erzeugt einen lehren Byte Array
			__declspec( dllexport ) Bytes();
            // Erzeugt einen neuen Byte Array mit bestimmter Lnge
            //  ln: Die Lnge des Arrays
			__declspec( dllexport ) Bytes( int ln );
            // Erzeugt ein neuen Byte Array durch kopieren der bytes
            //  daten: Die Bytes, die kopiert werden sollen
            //  ln: Die Anzahl der Bytes die kopiert werden sollen
			__declspec( dllexport ) Bytes( const char *daten, int ln );
			// Lscht den Array
			__declspec( dllexport ) ~Bytes();
			
            // Befllt den Bytearray durch kopieren der Bytes
            //  daten: Die Bytes, die kopiert werden sollen
			__declspec( dllexport ) void setBytes( const char *daten );
            // Lscht den Array und erzeugt einen neuen
            //  daten: Die Bytes, die kopiert werden sollen
            //  ln: Die nnge des Arrays
			__declspec( dllexport ) void setBytes( const char *daten, int ln );
            // Lscht den Array und bernimmt den bergebenen ohne zu kopieren
            //  daten: Der neue Byte Array
            //  ln: Die nnge des Arrays
			__declspec( dllexport ) void setBytesZ( char *daten, int ln );
            // Setzt alle Bytes des Arrays auf einen bestimmten Wert
            //  c: Der Wert, auf den die Bytes gesetzt werden sollen
			__declspec( dllexport ) void fll( const char c );
            // Setzt eine bestimmte Anzahl von Bytes des Arrays auf einen bestimmten Wert
            //  c: Der Wert, auf den die Bytes gesetzt werden sollen
            //  ln: Die Anzahl der Bytes die gesetzt werden sollen
			__declspec( dllexport ) void fll( const char c, int ln );
            // Setzt einen bestimmte Abschnitt von Bytes des Arrays auf einen bestimmten Wert
            //  c: Der Wert, auf den die Bytes gesetzt werden sollen
            //  beg: Die Startposition des zu setzenden Abschnittes
            //  end: Die Endposition des zu setzenden Abschnittes (nicht enthalten)
			__declspec( dllexport ) void fll( const char c, int beg, int end );
            // Kopiert bestimmte bytes in den Array
            //  c: Die Bytes, die kopiert werden sollen
            //  cLn: Die Anzahl an Bytes, die gesetzt werden sollen
			__declspec( dllexport ) void fll( const char *c, int cLn );
            // Setzt ein bestimmtes Byte auf einen Wert
            //  c: Der Wert, auf den das Byte gesetzt werden soll
            //  pos: Die Position des Bytes im Array
			__declspec( dllexport ) void set( const char c, int pos );
			
            // Gibt die Lnge des Arrays zurck
			__declspec( dllexport ) int getLnge() const;
            // Gibt den Array von Bytes zurck
			__declspec( dllexport ) char *getBytes() const;

            // Erhht den Reference Counting Zhler.
            //  return: this.
			__declspec( dllexport ) Bytes *getThis();
            // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
            //  return: 0.
			__declspec( dllexport ) Bytes *release();
		};

        // Kann Bytesequenzen mit bestimmten Schlsseln verschlsseln und entschlsseln
		class Schlssel
		{
		private:
			unsigned char *schlssel;
			int ln;
			int pos;
			int ref;

		public:
			// Erzeugt ein leeres Zeichnung
			__declspec( dllexport ) Schlssel();
            // Erzeugt ein neues Zeichnung mi einem Schlssel
            //  s: Der Schlssel, der zum verschlsseln und entchlsseln verwendet werden soll
            //  ln: Die Lnge des Schlssels
			__declspec( dllexport ) Schlssel( const char *s, int ln );
			// Lscht das Zeichnung
			__declspec( dllexport ) ~Schlssel();
			
            // Setzt die Position im Schlssel, wo mit dem verschlsseln und entschlsseln begonnen werden soll
			//  p: Die Position im Schlssel
            __declspec( dllexport ) void setPos( int p );
            // Setzt den Schlssel, der zum ver- und entschlsseln verwendet werden soll
            //  s: Der Schlssel
            //  ln: Die Lnge des Schlssels
			__declspec( dllexport ) void setSchlssel( const char *s, int ln );
            // Verschlsselt einen Byte Array mit dem gesetzten Schlssel
            //  daten: Der Byte Array, der verschlsselt werden soll. Wird von der Funktion verndert
			__declspec( dllexport ) void codieren( Bytes *daten );
            // Entschlsselt einen Byte Array mit dem gesetzten Schlssel
            //  daten: Der Byte Array, der entschlsselt werden soll. Wird von der Funktion verndert
			__declspec( dllexport ) void decodieren( Bytes *daten );
			
            // Erhht den Reference Counting Zhler.
            //  return: this.
			__declspec( dllexport ) Schlssel *getThis();
            // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
            //  return: 0.
			__declspec( dllexport ) Schlssel *release();
		};
	}
}

#endif